package lab;

import lab.storage.DbConnector;
import lab.storage.ScoreStorageInterface;

public class Setup {

	private static Setup instance;

	private ScoreStorageInterface scoreStorageInterface = new DbConnector();
	private double lochnessMinXPopsition;
	private double lochnessMinYPopsition;
	private double lochnessMinSpeed;
	private double lochnessMaxSpeed;
	private int lochnessMultiplier;
	private double boatCollisionHeight;
	private double boatHitPulseX;
	private double boatHitPulseYMin;
	private double boatHitPulseYMax;

	private Setup(ScoreStorageInterface scoreStorageInterface, double lochnessMinXPopsition,
			double lochnessMinYPopsition, double lochnessMinSpeed, double lochnessMaxSpeed, int lochnessMultiplier,
			double boatCollisionHeight, double boatHitPulseX, double boatHitPulseYMin, double boatHitPulseYMax) {
		this.scoreStorageInterface = scoreStorageInterface;
		this.lochnessMinXPopsition = lochnessMinXPopsition;
		this.lochnessMinYPopsition = lochnessMinYPopsition;
		this.lochnessMinSpeed = lochnessMinSpeed;
		this.lochnessMaxSpeed = lochnessMaxSpeed;
		this.lochnessMultiplier = lochnessMultiplier;
		this.boatCollisionHeight = boatCollisionHeight;
		this.boatHitPulseX = boatHitPulseX;
		this.boatHitPulseYMin = boatHitPulseYMin;
		this.boatHitPulseYMax = boatHitPulseYMax;
	}

	public static void configure(Setup setting) {
		instance = setting;
	}

	public static Setup getInstance() {
		return instance;
	}

	public ScoreStorageInterface getScoreStorageInterface() {
		return scoreStorageInterface;
	}

	public double getBoatCollisionHeight() {
		return boatCollisionHeight;
	}

	public double getBoatHitPulseX() {
		return boatHitPulseX;
	}

	public double getBoatHitPulseYMin() {
		return boatHitPulseYMin;
	}

	public double getBoatHitPulseYMax() {
		return boatHitPulseYMax;
	}

	public int getLochnessMultiplier() {
		return lochnessMultiplier;
	}

	public double getLochnessMinXPopsition() {
		return lochnessMinXPopsition;
	}

	public double getLochnessMinYPopsition() {
		return lochnessMinYPopsition;
	}

	public double getLochnessMinSpeed() {
		return lochnessMinSpeed;
	}

	public double getLochnessMaxSpeed() {
		return lochnessMaxSpeed;
	}

	public static Builder builder() {
		return new Builder();
	}

	public static Setup getInstanceForHardcoreGame() {
		return builder().lochnessMaxSpeed(500).lochnessMinSpeed(200).lochnessMultiplier(10).lochnessMinYPopsition(0.1)
				.build();
	}

	public static class Builder {
		private ScoreStorageInterface scoreStorageInterface = new DbConnector();
		private double lochnessMinXPopsition = 0.5;
		private double lochnessMinYPopsition = 0.5;
		private double lochnessMinSpeed = 50;
		private double lochnessMaxSpeed = 150;
		private int lochnessMultiplier = 1;
		private double boatCollisionHeight = 0.25;
		private double boatHitPulseX = 20;
		private double boatHitPulseYMin = 50;
		private double boatHitPulseYMax = 100;

		public Builder scoreStorageInterface(ScoreStorageInterface scoreStorageInterface) {
			this.scoreStorageInterface = scoreStorageInterface;
			return this;
		}

		public Builder lochnessMinXPopsition(double lochnessMinXPopsition) {
			this.lochnessMinXPopsition = lochnessMinXPopsition;
			return this;
		}

		public Builder lochnessMinYPopsition(double lochnessMinYPopsition) {
			this.lochnessMinYPopsition = lochnessMinYPopsition;
			return this;
		}

		public Builder lochnessMinSpeed(double lochnessMinSpeed) {
			this.lochnessMinSpeed = lochnessMinSpeed;
			return this;
		}

		public Builder lochnessMaxSpeed(double lochnessMaxSpeed) {
			this.lochnessMaxSpeed = lochnessMaxSpeed;
			return this;
		}

		public Builder lochnessMultiplier(int lochnessMultiplier) {
			this.lochnessMultiplier = lochnessMultiplier;
			return this;
		}

		public Builder boatCollisionHeight(double boatCollisionHeight) {
			this.boatCollisionHeight = boatCollisionHeight;
			return this;
		}

		public Builder boatHitPulseX(double boatHitPulseX) {
			this.boatHitPulseX = boatHitPulseX;
			return this;
		}

		public Builder boatHitPulseYMin(double boatHitPulseYMin) {
			this.boatHitPulseYMin = boatHitPulseYMin;
			return this;
		}

		public Builder boatHitPulseYMax(double boatHitPulseYMax) {
			this.boatHitPulseYMax = boatHitPulseYMax;
			return this;
		}

		public Setup build() {
			return new Setup(scoreStorageInterface, lochnessMinXPopsition, lochnessMinYPopsition, lochnessMinSpeed,
					lochnessMaxSpeed, lochnessMultiplier, boatCollisionHeight, boatHitPulseX, boatHitPulseYMin,
					boatHitPulseYMax);
		}
	}

}
